# (pre-)imaging script

thesteps = []
step_title = {0: 'velocity correction',
              1: 'continuum subtraction',
              2: 'SiO maser self cal, 1st round phase',
              3: 'SiO maser self cal, 2nd round phase and amp',
              4: 'Updated continuum subtraction',
              5: 'Image source continuum',
              6: 'Spw 3 SiO maser I,Q,U imaging and pola,poli creation',
              7: 'Spw 0 H2O maser phase selfcal',
              8: 'Spw 0 H2O maser amp selfcal',
              9: 'Spw 0 H2O maser cube I imaging',
              10: 'Export to fits'}

try:
    print 'List of steps to be executed ...', mysteps
    thesteps = mysteps
except:
    print 'global variable mysteps not set.'
  
if (thesteps==[]):
    thesteps = range(0,len(step_title))
    print 'Executing all steps: ', thesteps

import re

import os

ms1="uid___A002_Xb978c3_concatenated.ms.split.cal.polcal"

msbase="VYCMa.mstransform"
imagebase='VYCMa'
ms2="VYCMa.mstransform.ms"
ms3=ms2+".contsub"
contspw = "1:540~950,2:10~190;260~320;370~570;650~810;850~950,3:10~150;350~470"
contsub = "0:10~320;600~950,1:540~950,2:10~190;260~320;370~570;650~810;850~950,3:10~150;350~470,4:10~80;430~475"


mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Split out target and correct to constant velocity in target frame
# This is done in order to be able to perform self-calibration on 
# a single maser channel later

  spws=['0','1','2','3','4']
  catvis=[]
  for s in spws:
      os.system('rm -rf '+msbase+"_"+s+'.ms')
      mstransform(vis=ms1,
                outputvis=msbase+'_'+s+'.ms',
                spw=s,
                datacolumn='data',
                field='VY*',
                keepflags=F,
                regridms=T,
                outframe='lsrk')
      catvis.append(msbase+'_'+s+'.ms')

# concatenate corrected target data, as mstransform works on
# the individual spws

  os.system('rm -rf '+ms2)
  concat(vis=catvis,
  concatvis=ms2)
  os.system('rm -rf '+msbase+"_*")

mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# A first continuum subtraction

  clearcal(vis=ms2)

  os.system('rm -rf '+ms3+"*")
  uvcontsub3(vis=ms2,
             fitspw = contsub,
             combine = 'spw',
             field = 'VY*',
             spw = '')

mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# tlean one of the brightest, compact, SiO maser channels (270) 
# The mask is set manually closely around the maser peak

  os.system('rm -rf '+imagebase+'_spw3_270.clean*')
  os.system('rm -rf VYCMa-spw3_270.*')
  clearcal(vis=ms3)

  tclean(vis=ms3, 
         imagename=imagebase+'_spw3_270.clean',
         field='VY*', 
         spw='3', 
         specmode='cube', 
         start=270, 
         nchan=1,
         gridder='standard',
         imsize=512, 
         cell='0.05arcsec', 
         mask='circle[[259pix,262pix],5pix]', 
         weighting='briggs',
         robust=0.5, 
         niter=100, 
         cycleniter=10,
         savemodel='modelcolumn', 
         interactive=F)

# 70.21 / 0.42  S/N ~167

# One round of phase self calibration using 5 integrations

  gaincal(vis=ms3,
          field='VY*',
          caltable='VYCMa-spw3_270.p1',
          refant='DV12',
          spw='3:270',
          gaintype = 'T',
          calmode='p',
          solint='31s')

# Apply the solutions to all spw

  applycal(vis=ms3,
           gaintable='VYCMa-spw3_270.p1',
           spw='0,1,2,3,4',
           spwmap=[3,3,3,3,3],
           calwt=F,
           applymode='calonly')

  os.system('rm -rf '+imagebase+'_spw3_270_p1.clean*')
  tclean(vis=ms3, 
         imagename=imagebase+'_spw3_270_p1.clean',
         field='VY*', 
         spw='3', 
         specmode='cube', 
         start=270, 
         nchan=1,
         gridder='standard',
         imsize=512, 
         cell='0.05arcsec', 
         mask='circle[[259pix,262pix],5pix]', 
         weighting='briggs',
         robust=0.5, 
         niter=100, 
         cycleniter=10,
         savemodel='modelcolumn', 
         interactive=F)

# S/N 74.04/0.06 = 1234
# Note the significant improvement of the image

mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Use the new model to improve the phase self-calibration on
# integration level

  gaincal(vis=ms3,
          field='VY*',
          caltable='VYCMa-spw3_270.p2',
          refant='DV12',
          spw='3:270',
          gaintype = 'T',
          calmode='p',
          solint='int')

  applycal(vis=ms3,
           gaintable='VYCMa-spw3_270.p2',
           spw='0,1,2,3,4',
           spwmap=[3,3,3,3,3],
           calwt=F,
           applymode='calonly')

  os.system('rm -rf '+imagebase+'_spw3_270_p2.clean*')
  tclean(vis=ms3, 
         imagename=imagebase+'_spw3_270_p2.clean',
         field='VY*', 
         spw='3', 
         specmode='cube', 
         start=270, 
         nchan=1,
         gridder='standard',
         imsize=512, 
         cell='0.05arcsec',
         mask='circle[[259pix,262pix],5pix]', 
         weighting='briggs',
         robust=0.5, 
         niter=100, 
         cycleniter=50,
         savemodel='modelcolumn', 
         interactive=F)

#74.45/0.05 S/N 1489

# No significant further improvement. Now add an longer timescale amplitude
# Calibration

  gaincal(vis=ms3,
          field='VY*',
          caltable='VYCMa-spw3_270.a',
          refant='DV12',
          spw='3:270',
          gaintype = 'T',
          calmode='a',
          solint='31s',
          gaintable=['VYCMa-spw3_270.p2'])

  applycal(vis=ms3,
         gaintable=['VYCMa-spw3_270.p2','VYCMa-spw3_270.a'],
         spw='0,1,2,3,4',
         spwmap=[[3,3,3,3,3],[3,3,3,3,3]])

  os.system('rm -rf '+imagebase+'_spw3_270_ap.clean*')
  tclean(vis=ms3, 
         imagename=imagebase+'_spw3_270_ap.clean',
         field='VY*', 
         spw='3', 
         specmode='cube', 
         start=270, 
         nchan=1,
         gridder='standard',
         imsize=512, 
         cell='0.05arcsec',
         mask='circle[[259pix,262pix],5pix]', 
         weighting='briggs',
         robust=0.5, 
         niter=100, 
         cycleniter=50,
         savemodel='modelcolumn', 
         interactive=F)

# 74.43/0.018  4135

# Further significant improvement, which could be traced to a significant
# amplitude correction at one of the telescopes
 
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# apply the final self-calibration solutions to the full
# (non continuum-subtracted) data set and redo continuum subtraction 

  applycal(vis=ms2,
         gaintable=['VYCMa-spw3_270.p2','VYCMa-spw3_270.a'],
         spw='0,1,2,3,4',
         spwmap=[[3,3,3,3,3],[3,3,3,3,3]])

  os.system('rm -rf '+ms3+"*")
  uvcontsub3(vis=ms2,
             fitspw = contsub,
             combine = 'spw',
             field = 'VY*',
             spw = '')

mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Make a continuum image using the continuum channels away from the 
# emission lines and the H2O atmospheric window

  os.system('rm -rf VYCMa_B5.cont*')
  tclean(vis=ms2,
       imagename='VYCMa_B5.cont', 
       spw=contspw,               
       field='VY*',               
       specmode='mfs', 
       gridder='standard',
       deconvolver='clark',
       stokes='I',
       imsize=512,
       cell='0.05arcsec',
       mask='circle[[256pix,256pix],20pix]', 
       weighting='briggs',             
       robust=0.5,
       niter=100, 
       cycleniter=50,
       interactive=F)

mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Now image the SiO maser in spw 3
# Image I, Q and U separately allowing for negative thresholds for the
# polarisation images!

  os.system('rm -rf VYCMa.spw3*')
  tclean(vis=ms3,
       imagename='VYCMa.spw3.I', 
       spw='3',               
       field='VY*',
       specmode = 'cube',
       width=2,
       start='0',
       gridder='standard',
       deconvolver='clark',
       stokes='I',
       imsize=512,
       cell='0.05arcsec',
       weighting='briggs',             
       robust=0.5,
       mask='circle[[256pix,256pix],30pix]', 
       niter=50000, 
       cycleniter=50,
       threshold='0.05Jy',
       restfreq='172.481GHz',
       outframe = 'lsrk',
       veltype='radio',
       interactive=F)

# Q and U are cleaned using a mask based on the total intensity map

  immath(imagename='VYCMa.spw3.I.image',
         mode = 'evalexpr',
         outfile = 'VYCMa.spw3.mask',
         expr='iif(IM0>1.0,1.0,0.0)')

  tclean(vis=ms3,
       imagename='VYCMa.spw3.Q', 
       spw='3',               
       field='VY*',
       specmode = 'cube',
       width=2,
       start='0',
       gridder='standard',
       deconvolver='clarkstokes',
       stokes='Q',
       imsize=512,
       cell='0.05arcsec',
       weighting='briggs',             
       robust=0.5,
       mask='VYCMa.spw3.mask', 
       niter=10000, 
       cycleniter=50,
       threshold='-1Jy',
       cyclefactor=-10,
       minpsffraction=-10,
       restfreq='172.481GHz',
       outframe = 'lsrk',
       veltype='radio',
       interactive=F)

  tclean(vis=ms3,
       imagename='VYCMa.spw3.U', 
       spw='3',               
       field='VY*',
       specmode = 'cube',
       width=2,
       start='0',
       gridder='standard',
       deconvolver='clarkstokes',
       stokes='U',
       imsize=512,
       cell='0.05arcsec',
       weighting='briggs',             
       robust=0.5,
       mask='VYCMa.spw3.mask', 
       niter=10000, 
       cycleniter=50,
       threshold='-1Jy',
       cyclefactor=-10,
       minpsffraction=-10,
       restfreq='172.481GHz',
       outframe = 'lsrk',
       veltype='radio',
       interactive=F)

# Use immath to obtain polarised intensity and polarisation angle cubes

  immath(imagename=['VYCMa.spw3.Q.image/', 'VYCMa.spw3.U.image/'],
         mode="poli",
         outfile="VYCMa.spw3.poli.image",
         sigma="10.0mJy/beam")

  immath(imagename=['VYCMa.spw3.Q.image/', 'VYCMa.spw3.U.image/'],
         mode="pola",
         outfile="VYCMa.spw3.pola.image",
         polithresh="0.25Jy/beam")


mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Clean brightest H2O maser channel (532)
# Note that because of the atmospheric window, the SiO self-calibration 
# solutions were not optimal for the H2O.
# We will do H2O self-calibration. Note that the relative positions of the SiO
# and H2O will need to be carefully checked in this case.

  os.system('rm -rf '+imagebase+'_spw0_532.clean*')
  clearcal(vis=ms3)

  tclean(vis=ms3, 
         imagename=imagebase+'_spw0_532.clean',
         field='VY*', 
         spw='0', 
         specmode='cube', 
         start=532, 
         nchan=1,
         gridder='standard',
         imsize=512, 
         cell='0.05arcsec', 
         weighting='briggs',
         mask='circle[[258pix,261pix],5pix]', 
         robust=0.5, 
         niter=500,
         threshold='1.0Jy',
         cycleniter=10,
         savemodel='modelcolumn', 
         interactive=F)
# 683.2 / 4.2 = 163


# Only a single integration based phase self-calibration

  os.system('rm -rf VYCMa-spw0_532.*')
  gaincal(vis=ms3,
          field='VY*',
          caltable='VYCMa-spw0_532.p1',
          refant='DV12',
          spw='0:532',
          gaintype = 'T',
          calmode='p',
          solint='int')

# Apply only on the H2O spw itself.
# Note that we are using the continuum subtracted ms on which the SiO
# solutions were already applied. The H2O calibration solutions will thus
# add to the SiO solutions.

  applycal(vis=ms3,
           gaintable='VYCMa-spw0_532.p1',
           spw='0',
           spwmap=[0],
           calwt=F,
           applymode='calonly')

  os.system('rm -rf '+imagebase+'_spw0_532_p1.clean*')
  tclean(vis=ms3, 
         imagename=imagebase+'_spw0_532_p1.clean',
         field='VY*', 
         spw='0', 
         specmode='cube', 
         start=532, 
         nchan=1,
         gridder='standard',
         imsize=512, 
         cell='0.05arcsec', 
         weighting='briggs',
         mask='circle[[258pix,261pix],5pix]', 
         robust=0.5, 
         niter=750,
         threshold='0.75Jy',
         cycleniter=10,
         savemodel='modelcolumn', 
         interactive=F)

# S/N 693.9/2.4 = 289
# Some improvement in SNR

mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Also do an amplitude self-calibration. The atmospheric transmission, strongly
# elevation dependent, can have introduced an effect here.

  gaincal(vis=ms3,
          field='VY*',
          caltable='VYCMa-spw0_532.a',
          refant='DV12',
          spw='0:532',
          gaintype = 'T',
          calmode='a',
          solint='31s',
          gaintable=['VYCMa-spw0_532.p1'])

# apply both tables

  applycal(vis=ms3,
         gaintable=['VYCMa-spw0_532.p1','VYCMa-spw0_532.a'],
         spw='0',
         spwmap=[[0],[0]])

  os.system('rm -rf '+imagebase+'_spw0_532_ap.clean*')
  tclean(vis=ms3, 
         imagename=imagebase+'_spw0_532_ap.clean',
         field='VY*', 
         spw='0', 
         specmode='cube', 
         start=532, 
         nchan=1,
         gridder='standard',
         imsize=512, 
         cell='0.05arcsec',
         mask='circle[[258pix,261pix],5pix]', 
         weighting='briggs',
         robust=0.5, 
         niter=750,
         threshold='0.5Jy',
         cycleniter=10,
         savemodel='modelcolumn', 
         interactive=F)

# S/N 684.8/0.8 = 856
# A signficant improvement

mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Image the entire H2O spectral window in total intensity
# No polarisation imaging has been done. Linear polarisation might be 
# obtainable, but the leakage calibration in the atmospheric window 
# is sub-optimal

  os.system('rm -rf VYCMa.spw0*')
  tclean(vis=ms3,
       imagename='VYCMa.spw0', 
       spw='0',               
       field='VY*',
       specmode = 'cube',
       width=2,
       start='0',
       gridder='standard',
       deconvolver='clark',
       stokes='I',
       imsize=512,
       cell='0.05arcsec',
       weighting='briggs',             
       robust=0.5,    
       mask='circle[[256pix,256pix],30pix]', 
       niter=100000, 
       cycleniter=50,
       threshold='0.5Jy',
       restfreq='183.310GHz',
       outframe = 'lsrk',
       veltype='radio',
       interactive=F)

mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Do primary beam correction and export to fits

  imlist=['VYCMa.spw0','VYCMa.spw3.I','VYCMa.spw3.Q','VYCMa.spw3.U','VYCMa_B5.cont']

  for i in imlist:
      impbcor(imagename=i+'.image',
           pbimage=i+'.pb',
           outfile=i+'.pbcorr')
      exportfits(imagename=i+'.pbcorr',
              fitsimage=i+'.pbcorr.fits')
      exportfits(imagename=i+'.pb',
              fitsimage=i+'.pb.fits')
