# ALMA Data Imaging Script in full polarization


import os

thesteps = []
step_title = {0: 'Image the polarization calibrator',
              1: 'Image the bandpass calibrator',
              2: 'Image the phase calibrator'}


try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
  doall = raw_input('Execute the whole script? (Y)/N?')
  if 'y' in doall or 'Y' in doall:
    thesteps = range(0,len(step_title))
    print 'Executing all steps: ', thesteps
  else:
    raise Exception('\n\nSTOPPING SCRIPT!\n\n')


if re.search('^4.7.0', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.7.0')



msname = 'uid___A002_Xb978c3_concatenated.ms.split.cal.polcal'
fluxcal = '0'  # J0522-3627
bandpasscal = '1' # J2258-2758
phasecal = '2'  # J0747-3310
polcalib = '0' # J0522-3627
target = '3' # VY_CMa 
check = '4' # J0725-2640 


#############################
polcalibMask = ['box [ [ 470pix , 492pix] , [671pix, 571pix ] ]',
                'box [ [ 585pix , 558pix] , [763pix, 662pix ] ]',
                'box [ [ 188pix , 295pix] , [267pix, 363pix ] ]']

bandpasscalMask = 'box [ [505pix , 505pix] , [520pix, 520pix ] ]'

phasecalMask = 'box [ [505pix , 505pix] , [520pix, 520pix ] ]'

checkMask = 'box [ [505pix , 505pix] , [520pix, 520pix ] ]'

imsize = [1024]
cell = ['0.025arcsec']
#############################



niter = 2000
cycleniter = 100
weighting = 'natural'
robust = 0.5  # only used if weighting='briggs'
interactive = False
cyclefactor = -10.0 # Allow CLEANing negatives!
minpsffraction = -10.0 # Allow CLEANing negatives!

refant = 'DV12'

spw2clean = ['0','1','2']


mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for sp in spw2clean:
    os.system('rm -rf Pol_Calib_spw%i.*'%int(sp))
    tclean(vis=msname,
      field = polcalib,
      spw = str(sp),
      datacolumn = 'data',  
      mask = polcalibMask,
      imagename = 'Pol_Calib_spw%i'%int(sp),
      imsize = imsize,
      cell = cell,
      niter = 1000,
      threshold = '-1Jy',
      weighting = weighting,
      interactive = interactive,
      robust = robust,
      cyclefactor = cyclefactor,
      minpsffraction=minpsffraction,
      cycleniter = cycleniter,
      stokes = 'IQUV',
      deconvolver = 'clarkstokes',
      savemodel = 'modelcolumn',
      parallel = False)





mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for sp in spw2clean:
    os.system('rm -rf Bandpass_Calib_spw%i.*'%int(sp))
    tclean(vis=msname,
      field = bandpasscal,
      spw = str(sp),
      datacolumn = 'data',
      mask = bandpasscalMask,
      imagename = 'Bandpass_Calib_spw%i'%int(sp),
      imsize = imsize,
      cell = cell,
      niter = 1000,
      threshold = '-1Jy',
      weighting = weighting,
      interactive = interactive,
      robust = robust,
      cyclefactor = cyclefactor,
      minpsffraction=minpsffraction,
      cycleniter = cycleniter,
      stokes = 'IQUV',
      deconvolver = 'clarkstokes',
      savemodel = 'modelcolumn',
      parallel = False)


mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for sp in spw2clean:
    os.system('rm -rf Phase_Calib_spw%i.*'%int(sp))
    tclean(vis=msname,
      field = phasecal,
      spw = str(sp),
      datacolumn = 'data',
      mask = phasecalMask,
      imagename = 'Phase_Calib_spw%i'%int(sp),
      imsize = imsize,
      cell = cell,
      niter = 1000,
      threshold = '-1Jy',
      weighting = weighting,
      interactive = interactive,
      robust = robust,
      cyclefactor = cyclefactor,
      minpsffraction=minpsffraction,
      cycleniter = cycleniter,
      stokes = 'IQUV',
      deconvolver = 'clarkstokes',
      savemodel = 'modelcolumn',
      parallel = False)



