# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average'}

if 'applyonly' not in globals(): applyonly = True
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^4.7.0', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.7.0')


# CALIBRATE_AMPLI: 
# CALIBRATE_ATMOSPHERE: J0522-3627,J0747-3310,J2258-2758,VY_CMa
# CALIBRATE_BANDPASS: J2258-2758
# CALIBRATE_FLUX: J0522-3627
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0747-3310
# CALIBRATE_POINTING: J0522-3627,J2258-2758
# OBSERVE_CHECK: J0725-2640
# OBSERVE_TARGET: VY_CMa

# Using reference antenna = DA61

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_Xb978c3_X116b.ms') == False:
    importasdm('uid___A002_Xb978c3_X116b', asis='Antenna Station Receiver Source CalAtmosphere CalWVR CorrelatorMode SBSummary', bdfflags=True, lazy=False, process_caldevice=False)
    if not os.path.exists('uid___A002_Xb978c3_X116b.ms.flagversions'):
      print 'ERROR in importasdm. Output MS is probably not useful. Will stop here.'
      thesteps = []
  if applyonly != True: es.fixForCSV2555('uid___A002_Xb978c3_X116b.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_Xb978c3_X116b.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb978c3_X116b.ms.listobs')
  listobs(vis = 'uid___A002_Xb978c3_X116b.ms',
    listfile = 'uid___A002_Xb978c3_X116b.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_Xb978c3_X116b.ms',
    mode = 'manual',
    spw = '5~12,17~34',
    autocorr = T,
    flagbackup = F)


# BAD ANTENNA (HIGH TSYS):
  flagdata(vis = 'uid___A002_Xb978c3_X116b.ms',
    mode = 'manual',
    antenna = 'DA58',
    flagbackup = F)

  
  flagdata(vis = 'uid___A002_Xb978c3_X116b.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_Xb978c3_X116b.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_Xb978c3_X116b.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_Xb978c3_X116b.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb978c3_X116b.ms.wvr') 
  
  os.system('rm -rf uid___A002_Xb978c3_X116b.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xb978c3_X116b.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_Xb978c3_X116b.ms',
    caltable = 'uid___A002_Xb978c3_X116b.ms.wvr',
    spw = [25, 27, 29, 31, 33],
    smooth = '6.048s',
    toffset = 0,
    tie = ['J0725-2640,J0747-3310', 'VY_CMa,J0747-3310'],
    statsource = 'VY_CMa')
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xb978c3_X116b.ms.wvr', spw='25', antenna='DA61',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_Xb978c3_X116b.ms.wvr.plots/uid___A002_Xb978c3_X116b.ms.wvr') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb978c3_X116b.ms.tsys') 
  gencal(vis = 'uid___A002_Xb978c3_X116b.ms',
    caltable = 'uid___A002_Xb978c3_X116b.ms.tsys',
    caltype = 'tsys')
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_Xb978c3_X116b.ms.tsys',
    mode = 'manual',
    spw = '17:0~3;124~127,19:0~3;124~127,21:0~3;124~127,23:0~3;124~127',
    flagbackup = F)
  
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_Xb978c3_X116b.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True, showimage=False, 
    field='', figfile='uid___A002_Xb978c3_X116b.ms.tsys.plots.overlayTime/uid___A002_Xb978c3_X116b.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb978c3_X116b.ms.tsys', msName='uid___A002_Xb978c3_X116b.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Note: the correction for antenna DV21 is larger than 2mm.
  
  # Position for antenna DV21 is derived from baseline run made on 2016-10-18 07:20:40.
  
  # Position for antenna PM03 is derived from baseline run made on 2016-10-18 07:20:40.
  
  # Position for antenna DV12 is derived from baseline run made on 2016-10-18 07:20:40.
  
  # Position for antenna DA49 is derived from baseline run made on 2016-10-18 07:20:40.
  
  # Note: the correction for antenna DA61 is larger than 2mm.
  
  # Position for antenna DA61 is derived from baseline run made on 2016-10-18 07:20:40.
  
  # Note: the correction for antenna DA58 is larger than 2mm.
  
  # Position for antenna DA58 is derived from baseline run made on 2016-10-18 07:20:40.
  
  # Position for antenna DV05 is derived from baseline run made on 2016-10-18 07:20:40.
  
  # Note: the correction for antenna DV03 is larger than 2mm.
  
  # Position for antenna DV03 is derived from baseline run made on 2016-10-18 07:20:40.
  
  # Note: the correction for antenna DA42 is larger than 2mm.
  
  # Position for antenna DA42 is derived from baseline run made on 2016-10-18 07:20:40.
  
  os.system('rm -rf uid___A002_Xb978c3_X116b.ms.antpos') 
  gencal(vis = 'uid___A002_Xb978c3_X116b.ms',
    caltable = 'uid___A002_Xb978c3_X116b.ms.antpos',
    caltype = 'antpos',
    antenna = 'DA42,DA49,DA58,DA61,DV03,DV05,DV12,DV21,PM03',
 #   parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
    parameter = [3.18646e-03,-4.11829e-03,-6.27032e-04,1.99242e-05,-3.36634e-04,4.11058e-05,2.01626e-03,-8.13325e-03,-4.80618e-03,1.69212e-04,-4.73617e-03,-3.39150e-03,3.51720e-03,2.17003e-03,-2.39598e-03,-2.21858e-04,-4.05829e-04,3.15427e-04,-3.24672e-05,-2.71935e-04,1.65946e-04,1.65018e-03,-2.09331e-03,-1.01777e-03,1.34442e-04,-6.11351e-04,-2.26718e-05])
  
  
  # antenna x_offset y_offset z_offset total_offset baseline_date
  # DA58     2.01626e-03   -8.13325e-03   -4.80618e-03    9.65994e-03      2016-10-18 07:20:40
  # DA61     1.69212e-04   -4.73617e-03   -3.39150e-03    5.82771e-03      2016-10-18 07:20:40
  # DA42     3.18646e-03   -4.11829e-03   -6.27032e-04    5.24471e-03      2016-10-18 07:20:40
  # DV03     3.51720e-03    2.17003e-03   -2.39598e-03    4.77708e-03      2016-10-18 07:20:40
  # DV21     1.65018e-03   -2.09331e-03   -1.01777e-03    2.85323e-03      2016-10-18 07:20:40
  # PM03     1.34442e-04   -6.11351e-04   -2.26718e-05    6.26370e-04      2016-10-18 07:20:40
  # DV05    -2.21858e-04   -4.05829e-04    3.15427e-04    5.59833e-04      2016-10-18 07:20:40
  # DA49     1.99242e-05   -3.36634e-04    4.11058e-05    3.39719e-04      2016-10-18 07:20:40
  # DV12    -3.24672e-05   -2.71935e-04    1.65946e-04    3.20220e-04      2016-10-18 07:20:40
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_Xb978c3_X116b.ms', tsystable = 'uid___A002_Xb978c3_X116b.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_Xb978c3_X116b.ms',
    field = '0',
    spw = '25,27,29,31,33',
    gaintable = ['uid___A002_Xb978c3_X116b.ms.tsys', 'uid___A002_Xb978c3_X116b.ms.wvr', 'uid___A002_Xb978c3_X116b.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
# Not apply the wvr correction on the bandpass calibrator (unrealistic phase drift)  
  applycal(vis = 'uid___A002_Xb978c3_X116b.ms',
    field = '1',
    spw = '25,27,29,31,33',
    gaintable = ['uid___A002_Xb978c3_X116b.ms.tsys', 'uid___A002_Xb978c3_X116b.ms.antpos'],
    gainfield = ['1', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_Xb978c3_X116b.ms',
    field = '2',
    spw = '25,27,29,31,33',
    gaintable = ['uid___A002_Xb978c3_X116b.ms.tsys', 'uid___A002_Xb978c3_X116b.ms.wvr', 'uid___A002_Xb978c3_X116b.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  # Note: J0725-2640 didn't have any Tsys measurement, so I used the one made on VY_CMa. This is probably Ok.
  
  applycal(vis = 'uid___A002_Xb978c3_X116b.ms',
    field = '3',
    spw = '25,27,29,31,33',
    gaintable = ['uid___A002_Xb978c3_X116b.ms.tsys', 'uid___A002_Xb978c3_X116b.ms.wvr', 'uid___A002_Xb978c3_X116b.ms.antpos'],
    gainfield = ['4', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_Xb978c3_X116b.ms',
    field = '4',
    spw = '25,27,29,31,33',
    gaintable = ['uid___A002_Xb978c3_X116b.ms.tsys', 'uid___A002_Xb978c3_X116b.ms.wvr', 'uid___A002_Xb978c3_X116b.ms.antpos'],
    gainfield = ['4', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  if applyonly != True: es.getCalWeightStats('uid___A002_Xb978c3_X116b.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb978c3_X116b.ms.split') 
  os.system('rm -rf uid___A002_Xb978c3_X116b.ms.split.flagversions') 
  
  split(vis = 'uid___A002_Xb978c3_X116b.ms',
    outputvis = 'uid___A002_Xb978c3_X116b.ms.split',
    datacolumn = 'corrected',
    spw = '25,27,29,31,33',
    keepflags = T)
  
  


