# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'Concatenate the splitted MSs',
              1: 'Listobs, and save original flags',
              2: 'Initial flagging',
              3: 'Putting a model for the flux calibrator(s)',
              4: 'Save flags before bandpass cal',
              5: 'Bandpass calibration',
              6: 'Save flags before gain cal',
              7: 'Gain calibration',
              8: 'Save flags before applycal',
              9: 'Application of the bandpass and gain cal tables',
              10: 'Split out corrected column',
              11: 'Save flags after applycal'}

if 'applyonly' not in globals(): applyonly = True
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^4.7.0', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.7.0')


# CALIBRATE_AMPLI: 
# CALIBRATE_ATMOSPHERE: J0522-3627,J0747-3310,J2258-2758,VY_CMa
# CALIBRATE_BANDPASS: J2258-2758, J0635-7516
# CALIBRATE_FLUX: J0522-3627
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0747-3310
# CALIBRATE_POINTING: J0522-3627,J2258-2758
# OBSERVE_CHECK: J0725-2640
# OBSERVE_TARGET: VY_CMa

# Using reference antenna = DV12

# Concatenate the splitted MSs:
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  MSs = ['uid___A002_Xb978c3_X116b.ms.split',
         'uid___A002_Xb978c3_X1955.ms.split',
         'uid___A002_Xb978c3_X262a.ms.split']

  os.system('rm -rf concatenated.ms')
  os.system('rm -rf concatenated.ms.flagversions')

  concat(vis = MSs, 
         concatvis = 'uid___A002_Xb978c3_concatenated.ms.split', timesort=True)

print "# Calibration"

# Listobs, and save original flags
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf concatenated.ms.listobs')
  listobs(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    listfile = 'concatenated.ms.listobs')
  
  
  if not os.path.exists('uid___A002_Xb978c3_concatenated.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  flagdata(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  # Bad antennas
  flagdata(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    mode = 'manual',
    antenna = 'DV05,DA58',
    flagbackup = F)

# Putting a model for the flux calibrator(s)
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    standard = 'manual',
    field = 'J0522-3627',
    fluxdensity = [3.26775819918, 0, 0, 0],
    spix = -0.350805439176,
    reffreq = '177.065798354GHz')

# and for the bandpass calibrators for later gain transfer

  setjy(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    standard = 'manual',
    field = 'J2258-2758',
    fluxdensity = [1.575, 0, 0, 0],
    spix = -1.65,
    reffreq = '177.065798354GHz')

  setjy(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    standard = 'manual',
    field = 'J0635-7516',
    fluxdensity = [0.895, 0, 0, 0],
    spix = -1.2,
    reffreq = '177.065798354GHz')
  

# Save flags before bandpass cal
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass',
    field = '1,5', # J2258-2758 J0635-7516
    spw = '0:0~959,1:0~959,2:0~959,3:0~479,4:0~479',
    scan = '',
    solint = 'int',
    refant = 'DV12',
    calmode = 'p')
 
  gaincal(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf',
    field = '1,5', # J2258-2758  J0635-7516
    spw = '',
    scan = '',
    solint = 'inf',
    combine = 'scan,field,obs',
    refant = 'DV12',
    gaintable = 'uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass',
    spwmap = [[1,1,2,3,4]],
    calmode = 'p')

  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass')

  gaincal(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass',
    field = '1,5', # J2258-2758  J0635-7516
    spw = '0:0~959,1:0~959,2:0~959,3:0~479,4:0~479',
    scan = '',
    solint = 'int',
    refant = 'DV12',
    gaintable = 'uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf',
    calmode = 'p')

  if applyonly != True: es.checkCalTable('uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf', msName='uid___A002_Xb978c3_concatenated.ms.split', interactive=False) 

  if applyonly != True: es.checkCalTable('uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass', msName='uid___A002_Xb978c3_concatenated.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.bandpass') 

# a bandpass with solnorm=False to determine the amplitude offsets between
# spw for later transferal of spw1 solutions to spw0

  bandpass(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.bandpass',
    field = '1,5', # J2258-2758  J0635-7516
    scan = '',
    solint = 'inf',
    combine = 'scan,field,obs',
    refant = 'DV12',
    solnorm = False,
    bandtype = 'B',
    spwmap = [[1,1,2,3,4],[]],
    gaintable = ['uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass','uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf'])


  
  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch') 
 

  bandpass(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch',
    field = '1,5', # J2258-2758  J0635-7516
    scan = '',
    spw = '0',
    solint = 'inf,200ch',
    combine = 'scan,field,obs',
    refant = 'DV12',
    solnorm = False,
    bandtype = 'B',
    spwmap = [[1,1,2,3,4],[]],
    gaintable = ['uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass','uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf'])
 

  bandpass(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch',
    field = '1,5', # J2258-2758  J0635-7516
    scan = '',
    spw = '1~4',
    solint = 'inf,20ch',
    combine = 'scan,field,obs',
    refant = 'DV12',
    solnorm = False,
    append = T,
    bandtype = 'B',
    spwmap = [[1,1,2,3,4],[]],
    gaintable = ['uid___A002_Xb978c3_concatenated.ms.split.ap_pre_bandpass','uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf'])

  if applyonly != True: es.checkCalTable('uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch', msName='uid___A002_Xb978c3_concatenated.ms.split', interactive=False) 
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb978c3_concatenated.ms.split.bandpass', msName='uid___A002_Xb978c3_concatenated.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.phase_int') 



  gaincal(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.phase_int',
    field = '0~3,5', # J0522-3627,J2258-2758,J0747-3310,J0725-2640,J0635-7516
    solint = 'int',
    refant = 'DV12',
    gaintype = 'G',
    calmode = 'p',
    gaintable = ['uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch','uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb978c3_concatenated.ms.split.phase_int', msName='uid___A002_Xb978c3_concatenated.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.ampli_inf',
    field = '0~3,5', # J0522-3627,J2258-2758,J0747-3310,J0725-2640,J0635-7516
    solint = 'inf',
    refant = 'DV12',
    gaintype = 'T',
    calmode = 'a',
    spwmap = [[],[1,1,2,3,4],[]],
    gaintable = ['uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch', 'uid___A002_Xb978c3_concatenated.ms.split.phase_int','uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb978c3_concatenated.ms.split.ampli_inf', msName='uid___A002_Xb978c3_concatenated.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xb978c3_concatenated.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.ampli_inf',
    fluxtable = 'uid___A002_Xb978c3_concatenated.ms.split.flux_inf',
    refspwmap=[1,1,2,3,4],
    reference = '0') # J0522-3627
  
  casalog.setlogfile(mylogfile)

# Note that the flux values noted in the .fluxscale file of the
# calibrators used in setjy by design will be around 1.00
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_Xb978c3_concatenated.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_Xb978c3_concatenated.ms.split', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    caltable = 'uid___A002_Xb978c3_concatenated.ms.split.phase_inf',
    field = '0~3', # J0522-3627,J2258-2758,J0747-3310,J0725-2640
    solint = 'inf',
    refant = 'DV12',
    gaintype = 'G',
    calmode = 'p',
    gaintable = ['uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch','uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb978c3_concatenated.ms.split.phase_inf', msName='uid___A002_Xb978c3_concatenated.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0', '1']: # J0522-3627,J2258-2758
    applycal(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
      field = str(i),
      spwmap = [[],[],[1,1,2,3,4],[1,1,2,3,4]],
      gaintable = ['uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf','uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch', 'uid___A002_Xb978c3_concatenated.ms.split.phase_int', 'uid___A002_Xb978c3_concatenated.ms.split.flux_inf'],
      gainfield = ['','', i, i],
      interp = 'linear,linear,linear,linear',
      calwt = T,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    field = '2,3~4', # J0725-2640,VY_CMa
    spwmap = [[],[],[1,1,2,3,4],[1,1,2,3,4]],
    gaintable = ['uid___A002_Xb978c3_concatenated.ms.split.phasediff_inf','uid___A002_Xb978c3_concatenated.ms.split.bandpass_smooth20ch', 'uid___A002_Xb978c3_concatenated.ms.split.phase_inf', 'uid___A002_Xb978c3_concatenated.ms.split.flux_inf'],
    gainfield = ['','', '2', '2'], # J0747-3310
    interp = 'linear,linear,linear,linear',
    calwt = T,
    flagbackup = F)
  

# Split out corrected column
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.cal') 
  os.system('rm -rf uid___A002_Xb978c3_concatenated.ms.split.cal.flagversions') 
  
  listOfIntents = ['CALIBRATE_BANDPASS#ON_SOURCE',
   'CALIBRATE_FLUX#ON_SOURCE',
   'CALIBRATE_PHASE#ON_SOURCE',
   'CALIBRATE_POLARIZATION#ON_SOURCE',
   'CALIBRATE_WVR#ON_SOURCE',
   'OBSERVE_CHECK_SOURCE#ON_SOURCE',
   'OBSERVE_TARGET#ON_SOURCE']
  
  split(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    outputvis = 'uid___A002_Xb978c3_concatenated.ms.split.cal',
    datacolumn = 'corrected',
    intent = ','.join(listOfIntents),
    keepflags = T)
  
  

# Save flags after applycal
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb978c3_concatenated.ms.split',
    mode = 'save',
    versionname = 'AfterApplycal')
  
  

