# ALMA Data Reduction Script
# POLARIZATION CALIBRATION - VERSION DATE: 04-11-2016 - Rev. 6

# REVISED by gmoellen (2016Sep12) to calibrate the X/Y amp ratio
#  in the science target and phase calibrator

# REVISED by marti-vidal (Oct2016) for the B5 Science Verification
# (performs calibration by looping over spws, to avoid problems
# related to multi-resolution calibration tables in some 
# almahelper and almapolhelper functions).

# Calibration


thesteps = []
step_title = {0: 'Listobs and save original flags.',
              1: 'Additional flagging',
              2: 'Polarization calibration',
              3: 'Save flags before applycal',
              4: 'Apply polarization calibration',
              5: 'Save flags after applycal',
              6: 'Split out corrected column'}


try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re
import pylab as pl
import numpy as np
import os

import recipes.almahelpers as ah
import recipes.almapolhelpers as aph


if re.search('^4.7.0', casadef.casa_version) == None:
  sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.7.0')



# Using reference antenna = DV12


msname = 'uid___A002_Xb978c3_concatenated.ms.split.cal'
fluxcal = '0'  # J0522-3627
bandpasscal = '1' # J2258-2758
phasecal = '2'  # J0747-3310
polcalib = '0' # J0522-3627
target = '3,4' # VY_CMa and J0725-2640 (check source)

refant = 'DV12'

###################################################



mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  listobs(vis=msname, listfile='%s.listobs'%msname)
  if os.path.exists(msname+'.flagversions/flags.Original'):
    print 'Original flag versions already exists! Will not overwrite'
  else:
    flagmanager(vis=msname,mode='save',versionname='Original')


# Additional flagging:
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]









# Polarization calibration:
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  clearcal(vis=msname)


###########################
# Figure out typical scan duration of the polcalib scans:

  ms.open(msname)
  summary = ms.summary()
  ms.close()

  tb.open(msname)
  scnum = tb.getcol('SCAN_NUMBER')
  stid = tb.getcol('STATE_ID')
  tb.close()

  tb.open(msname+'/STATE')
  obsm = tb.getcol('OBS_MODE')
  tb.close()

  polst = [i for i in range(len(obsm)) if 'CALIBRATE_POLARIZATION' in obsm[i]]
  polscans = ['scan_%i'%i for i in np.unique(scnum[np.in1d(stid,polst)])]
  scandur = int(np.median([summary[ss]['0']['EndTime'] - summary[ss]['0']['BeginTime'] for ss in polscans])*86400.)
  print 'Will pre-average pol. data by %i seconds for calibration'%scandur

###########################




# Re-calibrate polarization calibrator:
  os.system('rm -rf %s.Gpol1'%msname)

  gaincal(vis = msname,
        caltable = '%s.Gpol1'%msname,
        field = polcalib,
        solint = 'inf',
        gaintype = 'G', 
        refant = refant)

  os.system('rm -rf %s.Gpol1.png'%msname)
  plotcal(msname+'.Gpol1','scan','amp',field='',poln='/',subplot=111, figfile=msname+'.Gpol1.png')



##########
# Save QU output in external file:
  orig_stdout = sys.stdout
  f = open('%s.QUfromGain.txt'%msname, 'w')
  sys.stdout = f

# Rough estimate of QU:
  qu = aph.qufromgain('%s.Gpol1'%msname)

  sys.stdout = orig_stdout
  f.close()

  f = open('%s.QUfromGain.txt'%msname)
  print f.read()
  f.close()
##########



##################################################
# We search for the scan where the polarization signal is minimum in XX and YY
# (i.e., maximum in XY and YX):

  tb.open('%s.Gpol1'%msname)
  scans = tb.getcol('SCAN_NUMBER')
  gains = np.squeeze(tb.getcol('CPARAM'))
  tb.close()
  scanlist = np.array(list(set(scans)))
  ratios = np.zeros(len(scanlist))
  for si, s in enumerate(scanlist):
    filt = scans == s
    ratio = np.sqrt(np.average(np.power(np.abs(gains[0,filt])/np.abs(gains[1,filt])-1.0,2.)))
    ratios[si] = ratio


  bestscidx = np.argmin(ratios)
  bestscan = scanlist[bestscidx]
  print 'Scan with highest expected X-Y signal: ', bestscan
#####################################################



# Cross-hand delay:
  os.system('rm -rf %s.Kcrs'%msname)
  gaincal(vis=msname,
        caltable='%s.Kcrs'%msname,
        selectdata=T,
        field = polcalib,
        scan=bestscan,
        gaintype='KCROSS',
        solint='inf',
        refant=refant, 
        smodel = [1,0,1,0],
        gaintable=['%s.Gpol1'%msname],
        interp=['linear'])



  orig_stdout = sys.stdout
  f = open('%s.PolFromGaincal.txt'%msname, 'w')
  sys.stdout = f


# Channel averaging for each spw:
  chav = ['inf,200ch','inf,20ch','inf','inf,20ch','inf,20ch']

# Loop over spws:
  for i in range(5):

# XY phase offset:
    os.system('rm -rf %s.XY0amb_spw%i'%(msname,i))
    gaincal(vis=msname,
          caltable='%s.XY0amb_spw%i'%(msname,i), 
          field= polcalib,
          spw=str(i),
          gaintype='XYf+QU',
          solint=chav[i],
          combine='scan,obs',
          preavg=scandur, 
          refant=refant,
          smodel=[1,0,1,0],
          gaintable=['%s.Gpol1'%msname,'%s.Kcrs'%msname],
          interp=['linear','nearest'])


  print 'END OF GAINCAL'

  sys.stdout = orig_stdout
  f.close()

  f = open('%s.PolFromGaincal.txt'%msname)
  print f.read()
  f.close()


# Solve QU ambiguity:
  S = []
  orig_stdout = sys.stdout
  f = open('%s.XY-Ambiguity.txt'%msname, 'w')
  sys.stdout = f

  for i in range(5):
    os.system('rm -rf %s.XY0'%msname)
    S.append(aph.xyamb(xytab='%s.XY0amb_spw%i'%(msname,i), qu=qu[0], xyout='%s.XY0_spw%i'%(msname,i)))

  sys.stdout = orig_stdout
  f.close()

  f = open('%s.XY-Ambiguity.txt'%msname)
  print f.read()
  f.close()


# Loop over spws:
  for i in range(5):

    plotcal(msname+'.XY0amb_spw%i'%i,'freq','phase',antenna='0',poln='X',
         plotrange=[0,0,-180,180],subplot=121)

    plotcal(msname+'.XY0_spw%i'%i,'freq','phase',antenna='0',poln='X',subplot=122,
         figfile=msname+'.XY0_spw%i_Amb-NoAmb.png'%i,plotrange=[0,0,-180,180])

    plotcal(msname+'.XY0_spw%i'%i,'chan','phase',antenna='0',poln='X',subplot=111,
         iteration='spw',plotrange=[0,0,-180,180],figfile=msname+'.XY_spw%i-CrossPhase.png'%i)


## Re-calibrate polarization calibrator (with right pol. model): 
  os.system('rm -rf %s.Gpol2'%msname) 

  gaincal(vis=msname,
        caltable='%s.Gpol2'%msname, 
        field=polcalib,
        solint='inf',
        refant=refant,
        gaintype = 'G', 
        smodel=S[2],
        parang=T)


  os.system('rm -rf %s.Gpol2.png'%msname)
  plotcal(msname+'.Gpol2','scan','amp',field='',poln='/',subplot=111, figfile=msname+'.Gpol2.png')


##########
## Check for any residual polarization signal:

  orig_stdout = sys.stdout
  f = open('%s.QUfromGain.txt'%msname, 'a')
  f.write('\n\n USING POLCAL MODEL:\n\n')
  sys.stdout = f

  qu2 = aph.qufromgain('%s.Gpol2'%msname)

  sys.stdout = orig_stdout
  f.close()

  f = open('%s.QUfromGain.txt'%msname)
  print f.read()
  f.close()
##########




# Plot RMS of gain ratios around 1.0:

  tb.open('%s.Gpol2'%msname)
  scans2 = tb.getcol('SCAN_NUMBER')
  gains = np.squeeze(tb.getcol('CPARAM'))
  tb.close()

  scanlist2 = np.array(list(set(scans2)))
  ratios2 = np.zeros(len(scanlist2))
  for si, s in enumerate(scanlist2):
    filt = scans2 == s
    ratio = np.sqrt(np.average(np.power(np.abs(gains[0,filt])/np.abs(gains[1,filt])-1.0,2.)))
    ratios2[si] = ratio


  os.system('rm -rf %s.GainRatiosPol.png'%msname)
  pl.ioff()
  fig = pl.gcf()
  pl.clf() #pl.figure()
  sub = fig.add_subplot(111)
  sub.plot(scanlist, ratios, 'or', label = 'No Polcal')
  sub.plot(scanlist2, ratios2, 'ob', label = 'Polcal')
  sub.plot([scanlist[bestscidx]], [ratios[bestscidx]], 'xk', label = 'Best Pol. Scan')
  sub.set_xlabel('Scan Number')
  sub.set_ylabel('Gain Ratio RMS')
  pl.legend(numpoints=1)
  pl.savefig('%s.GainRatiosPol.png'%msname)
  pl.clf()

  pl.ion()


# Calibrate D-terms:

  os.system('rm -rf %s.Df0*'%msname) 
  os.system('rm -rf %s.Gxyamp*'%msname)

# Loop over spws:
  for i in range(5):

    polcal(vis=msname,
       caltable='%s.Df0_spw%i'%(msname,i), 
       field= polcalib, 
       spw = str(i),
       solint=chav[i],
       combine='obs,scan',
       preavg=scandur,
       poltype='Dflls',
       refant='', #solve absolute D-term
       smodel=S[2],
       gaintable=['%s.Gpol2'%msname, '%s.Kcrs'%msname, '%s.XY0_spw%i'%(msname,i)],
       gainfield=['', '', ''],
       interp=['linear','nearest','linear'])

# Allow applying solutions to the parallel hands too:
    aph.Dgen(dtab='%s.Df0_spw%i'%(msname,i), dout='%s.Df0gen_spw%i'%(msname,i))


# THERE IS A BUG IN "dxy" when dealing with multi-resolution cal tables!
# commute Df0 from antenna to sky frame (for examination only)
 #   aph.dxy(dtab='%s.Df0_spw%i'%(msname,i), xytab='%s.XY0_spw%i'%(msname,i),dout='%s.Df0sky_spw%i'%(msname,i))
 #   plotcal(msname+'.Df0sky_spw%i'%i, 'freq','real', spw=str(i), figfile='%s.Dterm_sky_real_spw%i.png'%(msname,i))
 #   plotcal(msname+'.Df0sky_spw%i'%i, 'freq','imag', spw=str(i), figfile='%s.Dterm_sky_imag_spw%i.png'%(msname,i))

    plotcal(msname+'.Df0gen_spw%i'%i, 'freq','real', spw=str(i), figfile='%s.Dterm_real_spw%i.png'%(msname,i))
    plotcal(msname+'.Df0gen_spw%i'%i, 'freq','imag', spw=str(i), figfile='%s.Dterm_imag_spw%i.png'%(msname,i))



# solve for global normalized gain amp (to get X/Y ratios) on pol calibrator

# amp-only and normalized, so only X/Y amp ratios matter
    gaincal(vis=msname,
        caltable='%s.Gxyamp_spw%i'%(msname,i), 
        field=polcalib,
        spw = str(i),
        solint='inf',
        combine='scan,obs',
        refant=refant,
        gaintype='G',
        smodel=S[2],
        calmode='a',
        gaintable=['%s.Kcrs'%msname, '%s.XY0_spw%i'%(msname,i), '%s.Df0gen_spw%i'%(msname,i)],
        gainfield=['', '', ''],
        interp=['nearest','linear','linear'],
        solnorm=T,
        parang=T)

    plotcal(msname+'.Gxyamp_spw%i'%i, 'antenna','amp', spw='',
        iteration='spw',subplot=111,
        figfile='%s.Gxyamp_spw%i.png'%(msname,i))
    plotcal(msname+'.Gxyamp_spw%i'%i, 'antenna','amp', spw='',poln='/',
        iteration='spw',subplot=111,
        figfile='%s.GxyampRatio_spw%i.png'%(msname,i))


# Save flags before applycal
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = msname,
    mode = 'save',
    versionname = 'BeforeApplycal')
  

  


# Application of the polarization calibration
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Loop over sources and spws:
  for i in  set([phasecal, bandpasscal, target]): 
   for j in range(5):
    print 'Doing field %s and spw %i'%(i,j)
    applycal(vis = msname,
      field = str(i),
      spw = str(j),
      gaintable = ['%s.Gxyamp_spw%i'%(msname,j), 
                   '%s.Kcrs'%msname, '%s.XY0_spw%i'%(msname,j), '%s.Df0gen_spw%i'%(msname,j)],
      gainfield = [polcalib,polcalib, polcalib, polcalib],
      interp = ['nearest','nearest','nearest','nearest'],
      calwt = F,
      parang = T,
      flagbackup = F)
  
# Loop over sources and spws:
  for i in  [polcalib]:
   for j in range(5):
    print 'Doing field %s and spw %i'%(i,j)
    applycal(vis = msname,
      field = str(i),
      spw = str(j),
      gaintable = ['%s.Gpol2'%msname,'%s.Kcrs'%msname, '%s.XY0_spw%i'%(msname,j), '%s.Df0gen_spw%i'%(msname,j)],
      gainfield = [polcalib,polcalib, polcalib, polcalib],
      interp = ['linear','nearest','nearest','nearest'],
      calwt = F,
      parang = T,
      flagbackup = F)


# Save flags after applycal
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  flagmanager(vis = msname,
    mode = 'save',
    versionname = 'AfterApplycal')


# Split out corrected column
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf %s.polcal'%msname) 
  os.system('rm -rf %s.polcal.flagversions'%msname) 
  split(vis = msname,
    field = ','.join(set([target,phasecal,polcalib])),
    outputvis = '%s.polcal'%msname,
    datacolumn = 'corrected',
    keepflags = T)
  
  



